#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#define SEG_TIME 1041375600 // 1 de enero de 2003 a las 00:00:00 para time()
#define MAX_LIN 300 // --- Longitud máxima en caracteres

long NUM_PROBS;  // Número de problemas a generar
long NUM_USUS;	 // Número de usuarios a generar
long NUM_ENVIOS; // Número de envíos a generar

int *problemas;  // Identificadores de problemas
int *usuarios;   // Identificadores de usuarios


//----------------------- mostrar_sintaxis --------------------------

void mostrar_sintaxis(){
   printf("Sintaxis: generador Fich_fuente_problemas Fich_generar_problemas Num_problemas Fich_fuente_usuarios Fich_generar_usuarios Num_usuarios Fich_generar_envíos Num_envíos\n");
   printf("Parámetro 1: Nombre del fichero fuente de problemas\n");
   printf("Parámetro 2: Nombre del fichero de problemas a generar\n");
   printf("Parámetro 3: Número de problemas a generar\n");
   printf("Parámetro 4: Nombre del fichero fuente de usuarios\n");
   printf("Parámetro 5: Número del fichero de usuarios a generar\n");
   printf("Parámetro 6: Número de usuarios a generar\n");
   printf("Parámetro 7: Nombre del fichero de envíos a generar\n");
   printf("Parámetro 8: Número de envíos a generar\n\n");
}


// --------------------- Comprobación de parámetros ---------------------

bool comprobacion_param(int argc, char *argv[]){

   bool error=false;

  //--- Comprobación del número de parámetros
   if (argc != 9){
	printf("\nError: Número de parámetros incorrecto\n\n");
	mostrar_sintaxis();
	error=true;
    }
    else {

    //--- Comprobación existe fichero fuente de problemas
    if (access(argv[1], R_OK) == -1){
	printf("\nError: No existe el fichero fuente de problemas o no tiene permisos\n\n");
	error=true;
    }

   //--- Comprobación nº de problemas
    if (((NUM_PROBS=atol(argv[3])) == -1) || (NUM_PROBS<0)){
	printf("\nError: El parámetro 3 debe ser un número entero positivo  \n\n");
	error=true;
    }

    //--- Comprobación existe fichero fuente de usuarios
    if (access(argv[4], R_OK) == -1){
	printf("\nError: No existe el fichero fuente de usuarios o no tiene permisos\n\n");
	error=true;
    }

   //--- Comprobación nº de usuarios
    if (((NUM_USUS=atol(argv[6])) == -1) || (NUM_USUS<0)){
	printf("\nError: El parámetro 5 debe ser un número entero positivo  \n\n");
	error=true;
    }

   //--- Comprobación nº de envíos
    if (((NUM_ENVIOS=atol(argv[8])) == -1) || (NUM_USUS<0)){
	printf("\nError: El parámetro 5 debe ser un número entero positivo  \n\n");
	error=true;
    }

     if (!error) {

	printf("\nPrograma generador de juezonline. Versión 1.0 \n\n");

	// --- Muestra los parámetros por pantalla
	printf("----------Parámetros de la Ejecución ----------\n\n");
	printf("Fichero fuente de problemas: %s\n",argv[1]);
        printf("Fichero de problemas a generar: %s\n",argv[2]);
        printf("Nº de problemas: %d\n", NUM_PROBS);
        printf("Fichero fuente de usuarios: %s\n", argv[4]);
        printf("Fichero de usuarios a generar: %s\n",argv[5]);
        printf("Nº de usuarios: %d\n", NUM_USUS);
        printf("Fichero de envíos a generar: %s\n", argv[7]);
        printf("Nº de envíos: %d\n", NUM_ENVIOS);
	printf("-----------------------------------------------\n\n");
     }
     else
	mostrar_sintaxis();
   }
   return !error;
}

// ----------------------- aleatorio -----------------

long aleatorio(long rango){

   return (long) ( ((double) rand() /RAND_MAX) * rango);

}

// ------------------- contar_lineas ---------------------------

int contar_lineas(char *fichero){

      FILE *f;
      char linea[MAX_LIN];
      int conta=0;

      // Se recorre el fichero para contar línea a línea
      if ((f = fopen(fichero,"r")) != NULL){
        while((fgets(linea, MAX_LIN, f) != NULL))
		conta++;
	fclose(f);
	return conta;
	}
      else return -1;
}


// ---------------- generar_fich_problemas -----------------

int generar_fich_problemas(char *fich_fuente_prob, char *fich_generar_prob){

   FILE *f_fuente_prob;     //--- Fichero fuente de problemas
   FILE *f_generar_prob;    //--- Fichero de problemas a generar
   char linea[MAX_LIN];  //--- Auxiliar. Cada línea del fichero
   long conta_prob=0;  //--- Contador del número de problemas
   long num_max_prob;  //--- Número máximo de problemas a generar
   int i;		// --- Variable auxiliar
   int paso_lineas, salto_lineas;  // --- Saltar líneas aleatoriamente
  
   printf("Generando fichero de problemas...\n");

  // --- Se comprueba el número máximo de problemas que se puede generar
  if ((num_max_prob=contar_lineas(fich_fuente_prob)) < NUM_PROBS){
      printf("Error: El fichero fuente de problemas contiene un máximo de %d problemas a generar\n", num_max_prob);
      return 0;
    }
  else {
     problemas = new int [NUM_PROBS];  //--- Problemas almacenados

     paso_lineas = (int) (num_max_prob / NUM_PROBS); //Máximo salto de líneas

     //--- Lectura y generación del fichero de problemas
     if ((f_fuente_prob = fopen(fich_fuente_prob,"r")) != NULL){
       if ((f_generar_prob = fopen(fich_generar_prob,"w")) != NULL){	   
       while(conta_prob< NUM_PROBS){
	     // Salta un número de líneas aleatorio
     	     salto_lineas = (int) aleatorio(paso_lineas);
	     for (i=1; i<salto_lineas; i++)
		fgets(linea, MAX_LIN, f_fuente_prob);

	     // Lee una línea para guardar
	     if (fgets(linea, MAX_LIN, f_fuente_prob)!=NULL){
	        fprintf(f_generar_prob,"%s", linea); 
                problemas[conta_prob]= atol(strtok(linea,";"));
                conta_prob++;
		}
        }
        fclose(f_generar_prob);
      }
      fclose(f_fuente_prob);
    }
  printf("Fichero de problemas generado\n\n");
  return 1;
  }
}


// ---------------- generar_fich_usuarios -----------------

int generar_fich_usuarios(char *fich_fuente_usu, char *fich_generar_usu){

   FILE *f_fuente_usu;   //--- Fichero fuente de usuarios
   FILE *f_generar_usu;  //--- Fichero de problemas a usuarios
   char linea[MAX_LIN];  //--- Auxiliar. Cada línea del fichero
   long conta_usu=0;  	 //--- Contador del número de usuarios
   long num_max_usu;  	 //--- Número máximo de problemas a generar
   int i;  		 // variable auxiliar bucle for
   int paso_lineas, salto_lineas; // Variables auxiliares para aleatorio
   char id_usu[10];  	 // Identificador de usuario
   char nombre_usu[100]; // Nombre del usuario
   char nombre[50]; 	 // Primer nombre del usuario
   char fecha[25]; 	 // Fecha de alta
   char *paises[20]={"Spain","France", "United Kingdom", "United States", "Mexico", "India", "Italy", "Russia", "Japan", "Argentina", "Australia", "Korea", "Brazil", "Canada", "China", "Fiji", "Finland", "Germany", "South Africa", "Zimbabwe"};
   int probabilidad;  // variable auxiliar

   // --- Valores para el cálculo de la fecha aleatoria
   time_t n_seg=SEG_TIME;  // Número de segundos 
   time_t segundos; // Nª Segundos de cada mensaje
   int paso_seg= (int) ((365* (60*60*24) -1) / NUM_USUS);// Nº segundos que se incrementan para crear todos los usuarios en el año 2003
  
   printf("Generando fichero de usuarios...\n");

  // --- Se comprueba el número máximo de usuarios que se puede generar
  if ((num_max_usu=contar_lineas(fich_fuente_usu)) < NUM_USUS){
      printf("Error: El fichero fuente de usuarios contiene un máximo de %d usuarios a generar\n", num_max_usu);
      return 0;
    }
  else {
     usuarios = new int [NUM_USUS];  //--- Problemas almacenados

     paso_lineas = (int) (num_max_usu / NUM_USUS); //Máximo salto de líneas

     //--- Lectura y generación del fichero de usuarios
     if ((f_fuente_usu = fopen(fich_fuente_usu,"r")) != NULL){
       if ((f_generar_usu = fopen(fich_generar_usu,"w")) != NULL){	   
       while(conta_usu< NUM_USUS){

	   // Salta un número de líneas aleatorio
     	   salto_lineas = (int) aleatorio(paso_lineas);
	   for (i=1; i<salto_lineas; i++)
		fgets(linea, MAX_LIN, f_fuente_usu);

	   // Lee una línea para guardar
	   if (fgets(linea, MAX_LIN, f_fuente_usu)!=NULL){

	        // se corta el id_usu
		sprintf(id_usu,"%s",strtok(linea,";")); 
                usuarios[conta_usu]= atol(id_usu);

		// se corta el nombre del usu
		sprintf(nombre_usu, "%s", strtok(linea+strlen(id_usu)+1,"\n"));
		nombre_usu[strlen(nombre_usu)-1]='\0';

		// se genera la línea del usuario
		fprintf(f_generar_usu,"%s;%s", id_usu, nombre_usu);

		sprintf(nombre,"%s", strtok(nombre_usu," "));

		// se genera la página web (40% en blanco)
		probabilidad = aleatorio(100);
		if (probabilidad < 40)
		   fprintf(f_generar_usu,";", nombre);
                  else
		   fprintf(f_generar_usu,";www.%sweb.com", nombre);

		// se genera dirección correo electrónico (10% en blanco)
		probabilidad = aleatorio(100);
		if (probabilidad < 10)
		   fprintf(f_generar_usu,";", nombre);
                  else
		   fprintf(f_generar_usu,";%s@web.com", nombre);

		// se genera el país aleatoriamente
		i=aleatorio(20);
		fprintf(f_generar_usu,";%s", paises[i]);

     		// --- Cálculo de la FECHA
     		segundos =  n_seg + aleatorio(paso_seg);
     		strftime(fecha, 25, "%Y/%m/%d-%H:%M:%S", localtime(&segundos));
     		n_seg = n_seg + paso_seg;  //--- incremento de segundos
		fprintf(f_generar_usu,";[%s]\n", fecha);
                conta_usu++;
		}
        }

        fclose(f_generar_usu);
      }
      fclose(f_fuente_usu);
    }
  printf("Fichero de usuarios generado\n\n");
  return 1;
  }//else

}

/* ------------------ generar_fich_envios ------------- */

int generar_fich_envios(char *fich_generar_envios){

   FILE *f_envios;  //--- Fichero de envíos a generar
   long id_envio=5000000;
   long conta_envio=0;
   time_t n_seg=SEG_TIME+ (365*60*60*24);  // Número de segundos 1/1/2004
   time_t segundos; // Nª Segundos de cada mensaje
   int paso_seg= (int) ((3*365* (60*60*24) -1) / NUM_ENVIOS); // incremento
   char fecha[25]; 	 // Fecha de alta
   char *estados[10]={"AC","PE","WA", "CE", "RE", "TL", "ML", "OL", "RF", "OTHER"};
    int estado;    // estado
    int CPU;	   // tiempo CPU en milisegundos
    int memoria;   // ocupación memoria en kbytes
    long usuario;  // Id del usuario que envía
    long problema; // Id del problema del envío
    int fuente;    // Tipo código fuente
    char *fuentes[11]={"C","C++","Pascal", "Java"};
    int probabilidad;  // Variable auxiliar probabilidad

   printf("Generando fichero de envíos...\n");
  // se crea el fichero de envíos
  if ((f_envios = fopen(fich_generar_envios,"w")) != NULL){
   while (conta_envio < NUM_ENVIOS){

   // Generación del ID de Envío
	id_envio++;

   // Generación de la fecha
   segundos =  n_seg + aleatorio(paso_seg);
   strftime(fecha, 25, "%Y/%m/%d-%H:%M:%S", localtime(&segundos));
   n_seg = n_seg + paso_seg;  //--- incremento de segundos

   // Generación del Estado
   probabilidad = aleatorio(100);  
   if (probabilidad <= 29) estado=0;  //AC 29%
	else if (probabilidad <=34) estado=1; //PE 5%
		else if (probabilidad <=69) estado=2; //WA 35%
		else if (probabilidad <=79) estado=3; //CE 10%
		else if (probabilidad <=86) estado=4; //RE 7%
		else if (probabilidad <=94) estado=5; //TL 8%
		else if (probabilidad <=95) estado=6; //ML 1%
		else if (probabilidad <=96) estado=7; //OL 1%
		else if (probabilidad <=97) estado=8; //RF 4%
		else estado=9; //OTHER

   // Generación de la CPU
   CPU = aleatorio(20000);  // en milisegundos

   // Generación de la Memoria
   memoria = aleatorio(10000);  // en kbytes

   // Generación del ID de Usuario
   usuario = aleatorio(NUM_USUS);

   // Generación del Fuente
   probabilidad = aleatorio(100);  
   if (probabilidad <= 29) fuente=0;  //C 29%
	else if (probabilidad <=89) fuente=1; //C++ 60%
		else if (probabilidad <=96) fuente=2; //Pascal 7%
			else fuente=3; //Java 4%

   // Generación del ID del Problema
   problema = aleatorio(NUM_PROBS);

   // Se genera la línea del envío
   fprintf(f_envios, "%d;[%s];%s;%d;%d;%d;%s;%d\n", id_envio, fecha, estados[estado], CPU, memoria, usuarios[usuario], fuentes[fuente], problemas[problema]);

    conta_envio++;
   }

  printf("Fichero de envíos generado\n");
  return 1;
  }
  else return 0;
}

// -------------------------- liberar memoria ----------------
void liberar_memoria(){
        delete [] problemas;
	delete [] usuarios;
}

/* ---------------------- main ------------------------- */

main(int argc, char *argv[])
{
    // --- Comprobación de errores en los parámetros
    if (comprobacion_param(argc, argv)){  //--- Los parámetros son correctos

      // Generar ficheros	
      if (generar_fich_problemas(argv[1], argv[2]) && //generar problemas
          generar_fich_usuarios(argv[4], argv[5]) && //generar usuarios
          generar_fich_envios(argv[7])) //generar envíos
	  printf("\n--- La generación ha finalizado correctamente---\n\n");

	// liberar memoria
	liberar_memoria();

   } //if ... comprobación_param
} //main
	
